/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -N -m -i generictypes.h -p nm-ip6-configinterface /home/jgrulich/development/projects/libnm-qt/dbus/introspection/nm-ip6-config.xml
 *
 * qdbusxml2cpp is Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef NETWORKMANAGERQT_IP6_CONFIGINTERFACE_H
#define NETWORKMANAGERQT_IP6_CONFIGINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>
#include "generictypes.h"

/*
 * Proxy class for interface org.freedesktop.NetworkManager.IP6Config
 */
class OrgFreedesktopNetworkManagerIP6ConfigInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.NetworkManager.IP6Config"; }

public:
    OrgFreedesktopNetworkManagerIP6ConfigInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = 0);

    ~OrgFreedesktopNetworkManagerIP6ConfigInterface();

    Q_PROPERTY(IpV6DBusAddressList Addresses READ addresses)
    inline IpV6DBusAddressList addresses() const
    { return qvariant_cast< IpV6DBusAddressList >(property("Addresses")); }

    Q_PROPERTY(QStringList Domains READ domains)
    inline QStringList domains() const
    { return qvariant_cast< QStringList >(property("Domains")); }

    Q_PROPERTY(QString Gateway READ gateway)
    inline QString gateway() const
    { return qvariant_cast< QString >(property("Gateway")); }

    Q_PROPERTY(IpV6DBusNameservers Nameservers READ nameservers)
    inline IpV6DBusNameservers nameservers() const
    { return qvariant_cast< IpV6DBusNameservers >(property("Nameservers")); }

    Q_PROPERTY(IpV6DBusRouteList Routes READ routes)
    inline IpV6DBusRouteList routes() const
    { return qvariant_cast< IpV6DBusRouteList >(property("Routes")); }

    Q_PROPERTY(QStringList Searches READ searches)
    inline QStringList searches() const
    { return qvariant_cast< QStringList >(property("Searches")); }

public Q_SLOTS: // METHODS
Q_SIGNALS: // SIGNALS
    void PropertiesChanged(const QVariantMap &properties);
};

#endif
